@@ -9,6 +9,7 @@
 
 #include "widgetfactory.h"
 #include "KRatingWidget"
+#include "filemetadatautil_p.h"
 #include "kcommentwidget_p.h"
 #include "tagwidget.h"
 
@@ -95,51 +96,44 @@ QWidget *WidgetFactory::createWidget(const QString &prop, const QVariant &value,
     } else if (prop == QLatin1String("userComment")) {
         widget = createCommentWidget(value.toString(), parent);
     } else if (prop == QLatin1String("tags")) {
-        QStringList tags = value.toStringList();
-        QCollator coll;
-        coll.setNumericMode(true);
-        std::sort(tags.begin(), tags.end(), [&](const QString &s1, const QString &s2) {
-            return coll.compare(s1, s2) < 0;
-        });
-        widget = createTagWidget(tags, parent);
+        widget = createTagWidget(Baloo::Private::sortTags(value.toStringList()), parent);
     } else if (prop == QLatin1String("gpsLocation")) {
         const auto pair = value.value<QPair<float, float>>();
         const auto latitude = pair.first;
         const auto longitude = pair.second;
 
-        const QString geoUri = QStringLiteral("geo:%1,%2").arg(latitude).arg(longitude);
-
         const QString latitudeStr = latitude < 0 ? i18nc("Latitude (South)", "%1°S", -latitude) : i18nc("Latitude (North)", "%1°N", latitude);
         const QString longitudeStr = longitude < 0 ? i18nc("Longitude (West)", "%1°W", -longitude) : i18nc("Longitude (East)", "%1°E", longitude);
         const QString gpsLocationStr = latitudeStr + QLatin1Char(' ') + longitudeStr;
 
-        QLabel *valueWidget = createValueWidget(parent);
-
         if (const auto geoService = KApplicationTrader::preferredService(QStringLiteral("x-scheme-handler/geo"))) {
-            valueWidget->setTextFormat(Qt::RichText);
-            valueWidget->setTextInteractionFlags(Qt::TextBrowserInteraction);
+            const QString geoUri = QStringLiteral("geo:%1,%2").arg(latitude).arg(longitude);
+            QLabel *valueWidget = createLinkWidget(parent);
+
             valueWidget->setText(QStringLiteral("<a href='%1'>%2</a>").arg(geoUri, gpsLocationStr));
             valueWidget->setToolTip(i18nc("@info:tooltip Show location in map viewer", "Show location in %1", geoService->name()));
+            widget = valueWidget;
         } else {
+            QLabel *valueWidget = createValueWidget(parent);
             valueWidget->setText(gpsLocationStr);
+            widget = valueWidget;
         }
 
+    } else if (prop == QLatin1String("originUrl")) {
+        QLabel *valueWidget = createLinkWidget(parent);
+        QString valueString = value.toUrl().toString();
+        // Shrink link label
+        auto labelString = KStringHandler::csqueeze(valueString, maxUrlLength);
+        valueString = QStringLiteral("<a href=\"%1\">%2</a>").arg(valueString, labelString);
+        valueWidget->setText(valueString);
         widget = valueWidget;
+
     } else {
         QString valueString;
         QLabel *valueWidget = createValueWidget(parent);
 
         auto pi = KFileMetaData::PropertyInfo::fromName(prop);
-        if (pi.name() == QLatin1String("originUrl")) {
-            auto url = value.toUrl();
-            valueString = url.toString();
-            // Shrink link label
-            auto labelString = KStringHandler::csqueeze(valueString, maxUrlLength);
-            valueString = QStringLiteral("<a href=\"%1\">%2</a>").arg(valueString, labelString);
-            valueWidget->setTextFormat(Qt::RichText);
-            valueWidget->setTextInteractionFlags(Qt::TextBrowserInteraction);
-
-        } else if (pi.name() != QLatin1String("empty")) {
+        if (pi.property() != KFileMetaData::Property::Empty) {
             if (pi.valueType() == QVariant::DateTime || pi.valueType() == QVariant::Date) {
                 valueString = formatDateTime(value, m_dateFormat);
             } else {
@@ -166,7 +160,12 @@ QWidget *WidgetFactory::createTagWidget(const QStringList &tags, QWidget *parent
     tagWidget->setSelectedTags(tags);
 
     connect(tagWidget, &TagWidget::selectionChanged, this, &WidgetFactory::slotTagsChanged);
-    connect(tagWidget, &TagWidget::tagClicked, this, &WidgetFactory::slotTagClicked);
+    connect(tagWidget, &TagWidget::tagClicked, this, [this](const QString &tag) {
+        QUrl url;
+        url.setScheme(QStringLiteral("tags"));
+        url.setPath(tag);
+        Q_EMIT urlActivated(url);
+    });
 
     m_tagWidget = tagWidget;
     m_prevTags = tags;
@@ -232,11 +231,26 @@ QSize ValueWidget::sizeHint() const
 QLabel *WidgetFactory::createValueWidget(QWidget *parent)
 {
     auto valueWidget = new ValueWidget(parent);
-    valueWidget->setTextInteractionFlags(Qt::TextSelectableByMouse);
-    valueWidget->setTextFormat(Qt::PlainText);
     valueWidget->setWordWrap(true);
     valueWidget->setAlignment(Qt::AlignTop | Qt::AlignLeft);
-    connect(valueWidget, &ValueWidget::linkActivated, this, &WidgetFactory::slotLinkActivated);
+
+    valueWidget->setTextInteractionFlags(Qt::TextSelectableByMouse);
+    valueWidget->setTextFormat(Qt::PlainText);
+
+    return valueWidget;
+}
+
+QLabel *WidgetFactory::createLinkWidget(QWidget *parent)
+{
+    auto valueWidget = new ValueWidget(parent);
+    valueWidget->setWordWrap(true);
+    valueWidget->setAlignment(Qt::AlignTop | Qt::AlignLeft);
+
+    valueWidget->setTextInteractionFlags(Qt::TextBrowserInteraction);
+    valueWidget->setTextFormat(Qt::RichText);
+    connect(valueWidget, &ValueWidget::linkActivated, this, [this](const QString &url) {
+        Q_EMIT this->urlActivated(QUrl::fromUserInput(url));
+    });
 
     return valueWidget;
 }
@@ -255,8 +269,6 @@ void WidgetFactory::slotCommentChanged(const QString &comment)
         KFileMetaData::UserMetaData md(url.toLocalFile());
         md.setUserComment(comment);
     }
-    Q_EMIT dataChangeStarted();
-    Q_EMIT dataChangeFinished();
 }
 
 void WidgetFactory::slotRatingChanged(int rating)
@@ -269,8 +281,6 @@ void WidgetFactory::slotRatingChanged(int rating)
         KFileMetaData::UserMetaData md(url.toLocalFile());
         md.setRating(rating);
     }
-    Q_EMIT dataChangeStarted();
-    Q_EMIT dataChangeFinished();
 }
 
 void WidgetFactory::slotTagsChanged(const QStringList &tags)
@@ -297,29 +307,9 @@ void WidgetFactory::slotTagsChanged(const QStringList &tags)
         }
 
         m_prevTags = tags;
-        Q_EMIT dataChangeStarted();
-        Q_EMIT dataChangeFinished();
     }
 }
 
-//
-// Notifications
-//
-
-void WidgetFactory::slotLinkActivated(const QString &url)
-{
-    Q_EMIT urlActivated(QUrl::fromUserInput(url));
-}
-
-void WidgetFactory::slotTagClicked(const QString &tag)
-{
-    QUrl url;
-    url.setScheme(QStringLiteral("tags"));
-    url.setPath(tag);
-
-    Q_EMIT urlActivated(url);
-}
-
 //
 // Accessor Methods
 //
@@ -342,3 +332,5 @@ void Baloo::WidgetFactory::setDateFormat(const Baloo::DateFormats format)
 {
     m_dateFormat = static_cast<QLocale::FormatType>(format);
 }
+
+#include "moc_widgetfactory.cpp"
